@@ -2,27 +2,29 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
+
+static const char fileIdent[] = "$Id: editwin.c,v 1.12 2003/06/30 22:12:40 kretch Exp $";
 
 #define INCR 5000
 
-void owl_editwin_init(owl_editwin *e, WINDOW *win, int winlines, int wincols, int style) {
-  /* initialize the editwin e.
-   * 'win' is an already initialzed curses window that will be used by editwin
-   */
-  e->buff=malloc(INCR);
+/* initialize the editwin e.
+ * 'win' is an already initialzed curses window that will be used by editwin
+ */
+void owl_editwin_init(owl_editwin *e, WINDOW *win, int winlines, int wincols, int style, owl_history *hist)
+{
+  e->buff=owl_malloc(INCR);  
   e->buff[0]='\0';
   e->bufflen=0;
+  e->hist=hist;
   e->allocated=INCR;
   e->buffx=0;
   e->buffy=0;
   e->topline=0;
   e->winlines=winlines;
   e->wincols=wincols;
-  if (wincols > 80) {
-    e->fillcol=75;
-  } else {
-    e->fillcol=wincols-5;
-  }
+  e->fillcol=owl_editwin_limit_maxcols(wincols-1, owl_global_get_edit_maxfillcols(&g));
+  e->wrapcol=owl_editwin_limit_maxcols(wincols-1, owl_global_get_edit_maxwrapcols(&g));
   e->curswin=win;
   e->style=style;
   if ((style!=OWL_EDITWIN_STYLE_MULTILINE) &&
@@ -30,19 +32,63 @@ void owl_editwin_init(owl_editwin *e, WINDOW *win, int winlines, int wincols, in
     e->style=OWL_EDITWIN_STYLE_MULTILINE;
   }
   e->lock=0;
+  e->dotsend=0;
+  e->echochar='\0';
   if (win) werase(win);
 }
 
-void owl_editwin_set_curswin(owl_editwin *e, WINDOW *w, int winlines, int wincols) {
+void owl_editwin_set_curswin(owl_editwin *e, WINDOW *w, int winlines, int wincols)
+{
   e->curswin=w;
   e->winlines=winlines;
   e->wincols=wincols;
-  e->fillcol=wincols-5;
+  e->fillcol=owl_editwin_limit_maxcols(wincols-1, owl_global_get_edit_maxfillcols(&g));
+  e->wrapcol=owl_editwin_limit_maxcols(wincols-1, owl_global_get_edit_maxwrapcols(&g));
 }
 
-void owl_editwin_set_locktext(owl_editwin *e, char *text) {
-  /* set text to be 'locked in' at the beginning of the buffer, any
-     previous text (including locked text) will be overwritten */
+/* echo the character 'ch' for each normal character keystroke,
+ * excepting locktext.  This is useful for entering passwords etc.  If
+ * ch=='\0' characters are echo'd normally
+ */
+void owl_editwin_set_echochar(owl_editwin *e, int ch)
+{
+  e->echochar=ch;
+}
+
+WINDOW *owl_editwin_get_curswin(owl_editwin *e)
+{
+  return(e->curswin);
+}
+
+void owl_editwin_set_history(owl_editwin *e, owl_history *h)
+{
+  e->hist=h;
+}
+
+owl_history *owl_editwin_get_history(owl_editwin *e)
+{
+  return(e->hist);
+}
+
+void owl_editwin_set_dotsend(owl_editwin *e)
+{
+  e->dotsend=1;
+}
+
+int owl_editwin_limit_maxcols(int v, int maxv)
+{
+  if (maxv > 5 && v > maxv) {
+    return(maxv);
+  } else {
+    return(v);
+  }
+}
+
+/* set text to be 'locked in' at the beginning of the buffer, any
+ * previous text (including locked text) will be overwritten
+ */
+void owl_editwin_set_locktext(owl_editwin *e, char *text)
+{
   
   int x, y;
 
@@ -59,13 +105,16 @@ void owl_editwin_set_locktext(owl_editwin *e, char *text) {
   owl_editwin_redisplay(e, 0);
 }
 
-int owl_editwin_get_style(owl_editwin *e) {
+int owl_editwin_get_style(owl_editwin *e)
+{
   return(e->style);
 }
 
-void owl_editwin_new_style(owl_editwin *e, int newstyle) {
+void owl_editwin_new_style(owl_editwin *e, int newstyle, owl_history *h)
+{
   char *ptr;
-  
+
+  owl_editwin_set_history(e, h);
   if (e->style==newstyle) return;
 
   if (newstyle==OWL_EDITWIN_STYLE_MULTILINE) {
@@ -86,41 +135,51 @@ void owl_editwin_new_style(owl_editwin *e, int newstyle) {
   }
 }
 
-void owl_editwin_fullclear(owl_editwin *e) {
-  /* completly reinitialize the buffer */
-  free(e->buff);
-  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style);
+/* completly reinitialize the buffer */
+void owl_editwin_fullclear(owl_editwin *e)
+{
+  owl_free(e->buff);
+  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style, e->hist);
 }
 
-void owl_editwin_clear(owl_editwin *e) {
-  /* clear all text except for locktext and put the cursor at the beginning */
+/* clear all text except for locktext and put the cursor at the
+ * beginning
+ */
+void owl_editwin_clear(owl_editwin *e)
+{
+
   int lock;
+  int dotsend=e->dotsend;
   char *locktext=NULL;
-  
+
   lock=0;
   if (e->lock > 0) {
     lock=1;
 
-    locktext=malloc(e->lock+20);
+    locktext=owl_malloc(e->lock+20);
     strncpy(locktext, e->buff, e->lock);
     locktext[e->lock]='\0';
   }
 
-  free(e->buff);
-  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style);
+  owl_free(e->buff);
+  owl_editwin_init(e, e->curswin, e->winlines, e->wincols, e->style, e->hist);
 
   if (lock > 0) {
     owl_editwin_set_locktext(e, locktext);
   }
+  if (dotsend) {
+    owl_editwin_set_dotsend(e);
+  }
 
-  if (locktext) free(locktext);
+  if (locktext) owl_free(locktext);
   owl_editwin_adjust_for_locktext(e);
 }
 
 
-void _owl_editwin_addspace(owl_editwin *e) {
-  /* malloc more space for the buffer */
-  e->buff=realloc(e->buff, e->allocated+INCR);
+/* malloc more space for the buffer */
+void _owl_editwin_addspace(owl_editwin *e)
+{
+  e->buff=owl_realloc(e->buff, e->allocated+INCR);
   if (!e->buff) {
     /* error */
     return;
@@ -128,9 +187,17 @@ void _owl_editwin_addspace(owl_editwin *e) {
   e->allocated+=INCR;
 }
 
-void owl_editwin_redisplay(owl_editwin *e, int update) {
-  /* regenerate the text on the curses window */
-  /* if update == 1 then do a doupdate(), otherwise do not */
+void owl_editwin_recenter(owl_editwin *e)
+{
+  e->topline=e->buffy-(e->winlines/2);
+  if (e->topline<0) e->topline=0;
+  if (e->topline>owl_editwin_get_numlines(e)) e->topline=owl_editwin_get_numlines(e);
+}
+
+/* regenerate the text on the curses window */
+/* if update == 1 then do a doupdate(), otherwise do not */
+void owl_editwin_redisplay(owl_editwin *e, int update)
+{
   
   char *ptr1, *ptr2, *ptr3, *buff;
   int i;
@@ -166,25 +233,39 @@ void owl_editwin_redisplay(owl_editwin *e, int update) {
   }
   ptr3+=2;
 
-  buff=malloc(ptr3-ptr1+50);
+  buff=owl_malloc(ptr3-ptr1+50);
   strncpy(buff, ptr1, ptr3-ptr1);
   buff[ptr3-ptr1]='\0';
-  waddstr(e->curswin, buff);
+  if (e->echochar=='\0') {
+    waddstr(e->curswin, buff);
+  } else {
+    /* translate to echochar, *except* for the locktext */
+    int len;
+    int dolocklen=e->lock-(ptr1-e->buff);
+
+    for (i=0; i<dolocklen; i++) {
+      waddch(e->curswin, buff[i]);
+    }
+    len=strlen(buff);
+    for (i=0; i<len-dolocklen; i++) {
+      waddch(e->curswin, e->echochar);
+    }
+  }
   wmove(e->curswin, e->buffy-e->topline, e->buffx);
   wnoutrefresh(e->curswin);
   if (update==1) {
     doupdate();
   }
-  free(buff);
+  owl_free(buff);
 }
 
 
-int _owl_editwin_linewrap_word(owl_editwin *e) {
-  /* linewrap the word just before the cursor.
-   * returns 0 on success
-   * returns -1 if we could not wrap.
-   */
-
+/* linewrap the word just before the cursor.
+ * returns 0 on success
+ * returns -1 if we could not wrap.
+ */
+int _owl_editwin_linewrap_word(owl_editwin *e)
+{
   int i, z;
 
   z=_owl_editwin_get_index_from_xy(e);
@@ -196,6 +277,7 @@ int _owl_editwin_linewrap_word(owl_editwin *e) {
       e->buff[i]='\n';
       e->buffy++;
       e->buffx=z-i-1;
+      /* were we on the last line */
       return(0);
     } else if (e->buff[i]=='\n' || i<=e->lock) {
       /* we hit the begginning of the line or the buffer, we cannot
@@ -206,9 +288,11 @@ int _owl_editwin_linewrap_word(owl_editwin *e) {
   }
 }
 
-void owl_editwin_insert_char(owl_editwin *e, char c) {
-  /* insert a character at the current point (shift later
-   * characters over) */
+/* insert a character at the current point (shift later
+ * characters over)
+ */
+void owl_editwin_insert_char(owl_editwin *e, char c)
+{
   
   int z, i, ret;
 
@@ -231,8 +315,8 @@ void owl_editwin_insert_char(owl_editwin *e, char c) {
   /* get the insertion point */
   z=_owl_editwin_get_index_from_xy(e);
 
-  /* are we going to insert at the last column? If it's a '\n' we let it slide */
-  if ((e->buffx+1==e->wincols) && (c!='\n')) {
+  /* If we're going to insert at the last column do word wrapping, unless it's a \n */
+  if ((e->buffx+1==e->wrapcol) && (c!='\n')) {
     ret=_owl_editwin_linewrap_word(e);
     if (ret==-1) {
       /* we couldn't wrap, insert a hard newline instead */
@@ -262,8 +346,9 @@ void owl_editwin_insert_char(owl_editwin *e, char c) {
   }
 }
 
-void owl_editwin_overwrite_char(owl_editwin *e, char c) {
-  /* overwrite the character at the current point with 'c' */
+/* overwrite the character at the current point with 'c' */
+void owl_editwin_overwrite_char(owl_editwin *e, char c)
+{
   int z;
   
   /* \r is \n */
@@ -304,10 +389,11 @@ void owl_editwin_overwrite_char(owl_editwin *e, char c) {
 
 }
 
-void owl_editwin_delete_char(owl_editwin *e) {
-  /* delete the character at the current point, following chars
-   * shift left.
-   */ 
+/* delete the character at the current point, following chars
+ * shift left.
+ */ 
+void owl_editwin_delete_char(owl_editwin *e)
+{
   int z, i;
 
   if (e->bufflen==0) return;
@@ -315,6 +401,8 @@ void owl_editwin_delete_char(owl_editwin *e) {
   /* get the deletion point */
   z=_owl_editwin_get_index_from_xy(e);
 
+  if (z==e->bufflen) return;
+
   for (i=z; i<e->bufflen; i++) {
     e->buff[i]=e->buff[i+1];
   }
@@ -322,10 +410,42 @@ void owl_editwin_delete_char(owl_editwin *e) {
   e->buff[e->bufflen]='\0';
 }
 
-void owl_editwin_insert_string(owl_editwin *e, char *string) {
-  /* insert 'string' at the current point, later text is shifted
-   * right
-   */
+/* Swap the character at point with the character at point-1 and
+ * advance the pointer.  If point is at beginning of buffer do
+ * nothing.  If point is after the last character swap point-1 with
+ * point-2.  (Behaves as observed in tcsh and emacs).  
+ */
+void owl_editwin_transpose_chars(owl_editwin *e)
+{
+  int z;
+  char tmp;
+
+  if (e->bufflen==0) return;
+  
+  /* get the cursor point */
+  z=_owl_editwin_get_index_from_xy(e);
+
+  if (z==e->bufflen) {
+    /* point is after last character */
+    z--;
+  }  
+
+  if (z-1 < e->lock) {
+    /* point is at beginning of buffer, do nothing */
+    return;
+  }
+
+  tmp=e->buff[z];
+  e->buff[z]=e->buff[z-1];
+  e->buff[z-1]=tmp;
+  owl_editwin_key_right(e);
+}
+
+/* insert 'string' at the current point, later text is shifted
+ * right
+ */
+void owl_editwin_insert_string(owl_editwin *e, char *string)
+{
   int i, j;
 
   j=strlen(string);
@@ -334,11 +454,13 @@ void owl_editwin_insert_string(owl_editwin *e, char *string) {
   }
 }
 
-void owl_editwin_overwrite_string(owl_editwin *e, char *string) {
+/* write 'string' at the current point, overwriting text that is
+ * already there
+ */
+
+void owl_editwin_overwrite_string(owl_editwin *e, char *string)
+{
   int i, j;
-  /* write 'string' at the current point, overwriting text that is
-   * already there
-   */
 
   j=strlen(string);
   for (i=0; i<j; i++) {
@@ -346,10 +468,11 @@ void owl_editwin_overwrite_string(owl_editwin *e, char *string) {
   }
 }
 
-int _owl_editwin_get_index_from_xy(owl_editwin *e) {
-  /* get the index into e->buff for the current cursor
-   * position.
-   */
+/* get the index into e->buff for the current cursor
+ * position.
+ */
+int _owl_editwin_get_index_from_xy(owl_editwin *e)
+{
   int i;
   char *ptr1, *ptr2;
 
@@ -382,7 +505,8 @@ int _owl_editwin_get_index_from_xy(owl_editwin *e) {
   return(ptr1-e->buff);
 }
 
-void _owl_editwin_set_xy_by_index(owl_editwin *e, int index) {
+void _owl_editwin_set_xy_by_index(owl_editwin *e, int index)
+{
   int z, i;
 
   z=_owl_editwin_get_index_from_xy(e);
@@ -397,15 +521,17 @@ void _owl_editwin_set_xy_by_index(owl_editwin *e, int index) {
   }
 }
 
-void owl_editwin_adjust_for_locktext(owl_editwin *e) {
+void owl_editwin_adjust_for_locktext(owl_editwin *e)
+{
   /* if we happen to have the cursor over locked text
    * move it to be out of the locktext region */
-  while (_owl_editwin_get_index_from_xy(e)<e->lock) {
-    owl_editwin_key_right(e);
+  if (_owl_editwin_get_index_from_xy(e)<e->lock) {
+    _owl_editwin_set_xy_by_index(e, e->lock);
   }
 }
 
-void owl_editwin_backspace(owl_editwin *e) {
+void owl_editwin_backspace(owl_editwin *e)
+{
   /* delete the char before the current one
    * and shift later chars left
    */
@@ -416,7 +542,8 @@ void owl_editwin_backspace(owl_editwin *e) {
   owl_editwin_adjust_for_locktext(e);
 }
 
-void owl_editwin_key_up(owl_editwin *e) {
+void owl_editwin_key_up(owl_editwin *e)
+{
   if (e->buffy > 0) e->buffy--;
   if (e->buffx >= owl_editwin_get_numchars_on_line(e, e->buffy)) {
     e->buffx=owl_editwin_get_numchars_on_line(e, e->buffy);
@@ -430,7 +557,8 @@ void owl_editwin_key_up(owl_editwin *e) {
   owl_editwin_adjust_for_locktext(e);
 }
 
-void owl_editwin_key_down(owl_editwin *e) {
+void owl_editwin_key_down(owl_editwin *e)
+{
   /* move down if we can */
   if (e->buffy+1 < owl_editwin_get_numlines(e)) e->buffy++;
 
@@ -448,7 +576,8 @@ void owl_editwin_key_down(owl_editwin *e) {
   owl_editwin_adjust_for_locktext(e);
 }
 
-void owl_editwin_key_left(owl_editwin *e) {
+void owl_editwin_key_left(owl_editwin *e)
+{
   /* move left if we can, and maybe up a line */
   if (e->buffx>0) {
     e->buffx--;
@@ -466,14 +595,16 @@ void owl_editwin_key_left(owl_editwin *e) {
   owl_editwin_adjust_for_locktext(e);
 }
 
-void owl_editwin_key_right(owl_editwin *e) {
+void owl_editwin_key_right(owl_editwin *e)
+{
   int i;
 
   /* move right if we can, and skip down a line if needed */
   i=owl_editwin_get_numchars_on_line(e, e->buffy);
   if (e->buffx < i) {
     e->buffx++;
-  } else if (e->buffy+1 <= owl_editwin_get_numlines(e)) {
+    /*  } else if (e->buffy+1 < owl_editwin_get_numlines(e)) { */
+  } else if (_owl_editwin_get_index_from_xy(e) < e->bufflen) {
     if (e->style==OWL_EDITWIN_STYLE_MULTILINE) {
       e->buffx=0;
       e->buffy++;
@@ -486,7 +617,8 @@ void owl_editwin_key_right(owl_editwin *e) {
   }
 }
 
-void owl_editwin_move_to_nextword(owl_editwin *e) {
+void owl_editwin_move_to_nextword(owl_editwin *e)
+{
   int i, x;
 
   /* if we're starting on a space, find the first non-space */
@@ -522,8 +654,10 @@ void owl_editwin_move_to_nextword(owl_editwin *e) {
   }
 }
 
-void owl_editwin_move_to_previousword(owl_editwin *e) {
-  /* go backwards to the last non-space character */
+/* go backwards to the last non-space character
+ */
+void owl_editwin_move_to_previousword(owl_editwin *e)
+{
   int i, x;
 
   /* are we already at the beginning of the word? */
@@ -557,7 +691,8 @@ void owl_editwin_move_to_previousword(owl_editwin *e) {
 }
 
 
-void owl_editwin_delete_nextword(owl_editwin *e) {
+void owl_editwin_delete_nextword(owl_editwin *e)
+{
   int z;
 
   if (e->bufflen==0) return;
@@ -582,7 +717,21 @@ void owl_editwin_delete_nextword(owl_editwin *e) {
   owl_editwin_delete_char(e);
 }
 
-void owl_editwin_delete_to_endofline(owl_editwin *e) {
+void owl_editwin_delete_previousword(owl_editwin *e)
+{
+  /* go backwards to the last non-space character, then delete chars */
+  int i, startpos, endpos;
+
+  startpos = _owl_editwin_get_index_from_xy(e);
+  owl_editwin_move_to_previousword(e);
+  endpos = _owl_editwin_get_index_from_xy(e);
+  for (i=0; i<startpos-endpos; i++) {
+    owl_editwin_delete_char(e);
+  }
+}
+
+void owl_editwin_delete_to_endofline(owl_editwin *e)
+{
   int i;
 
   if (owl_editwin_get_numchars_on_line(e, e->buffy)>e->buffx) {
@@ -591,6 +740,8 @@ void owl_editwin_delete_to_endofline(owl_editwin *e) {
     while(i < e->bufflen) {
       if (e->buff[i]!='\n') {
 	owl_editwin_delete_char(e);
+      } else if ((e->buff[i]=='\n') && (i==e->bufflen-1)) {
+	owl_editwin_delete_char(e);
       } else {
 	return;
       }
@@ -602,10 +753,23 @@ void owl_editwin_delete_to_endofline(owl_editwin *e) {
   }
 }
 
-void owl_editwin_move_to_end(owl_editwin *e) {
+void owl_editwin_move_to_line_end(owl_editwin *e)
+{
+  e->buffx=owl_editwin_get_numchars_on_line(e, e->buffy);
+}
+
+void owl_editwin_move_to_line_start(owl_editwin *e)
+{
+  e->buffx=0;
+  owl_editwin_adjust_for_locktext(e);
+}
+
+void owl_editwin_move_to_end(owl_editwin *e)
+{
   /* go to last char */
   e->buffy=owl_editwin_get_numlines(e)-1;
   e->buffx=owl_editwin_get_numchars_on_line(e, e->buffy);
+  owl_editwin_key_right(e);
 
   /* do we need to scroll? */
   if (e->buffy-e->topline > e->winlines) {
@@ -613,18 +777,18 @@ void owl_editwin_move_to_end(owl_editwin *e) {
   }
 }
 
-void owl_editwin_move_to_top(owl_editwin *e) {
+void owl_editwin_move_to_top(owl_editwin *e)
+{
   _owl_editwin_set_xy_by_index(e, 0);
 
   /* do we need to scroll? */
-  if (e->buffy-e->topline < 0) {
-    e->topline-=e->winlines/2;
-  }
+  e->topline=0;
 
   owl_editwin_adjust_for_locktext(e);
 }
 
-void owl_editwin_fill_paragraph(owl_editwin *e) {
+void owl_editwin_fill_paragraph(owl_editwin *e)
+{
   int i, save;
 
   /* save our starting point */
@@ -686,66 +850,63 @@ void owl_editwin_fill_paragraph(owl_editwin *e) {
   }
 }
 
+/* returns true if only whitespace remains */
+int owl_editwin_is_at_end(owl_editwin *e)
+{
+  int cur=_owl_editwin_get_index_from_xy(e);
+  return (only_whitespace(e->buff+cur));
+}
 
-void owl_editwin_process_char(owl_editwin *e, int j) {
+int owl_editwin_check_dotsend(owl_editwin *e)
+{
+  int i;
+
+  if (!e->dotsend) return(0);
+  for (i=e->bufflen-1; i>0; i--) {
+    if (e->buff[i] == '.' 
+	&& (e->buff[i-1] == '\n' || e->buff[i-1] == '\r')
+	&& (e->buff[i+1] == '\n' || e->buff[i+1] == '\r')) {
+      e->bufflen = i;
+      e->buff[i] = '\0';
+      return(1);
+    }
+    if (!isspace((int) e->buff[i])) {
+      return(0);
+    }
+  }
+  return(0);
+}
+
+void owl_editwin_post_process_char(owl_editwin *e, int j)
+{
+  /* check if we need to scroll down */
+  if (e->buffy-e->topline >= e->winlines) {
+    e->topline+=e->winlines/2;
+  }
+  if ((j==13 || j==10) && owl_editwin_check_dotsend(e)) {
+    owl_command_editmulti_done(e);
+    return;
+  }
+  owl_editwin_redisplay(e, 0);  
+}
+
+void owl_editwin_process_char(owl_editwin *e, int j)
+{
   if (j == ERR) return;
-
-  /* meta keys */
-  if (j == META('f')) {
-    owl_editwin_move_to_nextword(e);
-  } else if (j == META('b')) {
-    owl_editwin_move_to_previousword(e);
-  } else if (j == META('<')) {
-    owl_editwin_move_to_top(e);
-  } else if (j == META('>')) {
-    owl_editwin_move_to_end(e);
-  } else if (j == META('d')) {
-    owl_editwin_delete_nextword(e);
-  } else if (j == META('q')) {
-    owl_editwin_fill_paragraph(e);
-  } else if ((j==KEY_UP) || (j==CTRL('p'))) {
-    if (e->style==OWL_EDITWIN_STYLE_ONELINE) return;
-    owl_editwin_key_up(e);
-  } else if ((j==KEY_DOWN) || (j==CTRL('n'))) {
-    if (e->style==OWL_EDITWIN_STYLE_ONELINE) return;
-    owl_editwin_key_down(e);
-  } else if ((j==KEY_RIGHT) || (j==CTRL('f'))) {
-    owl_editwin_key_right(e);
-  } else if ((j==KEY_LEFT) || (j==CTRL('b'))) {
-    owl_editwin_key_left(e);
-  } else if (j==CTRL('h') || j==127 || j==KEY_BACKSPACE || j==KEY_DC) {
-    owl_editwin_backspace(e);
-  } else if (j==CTRL('u')) {
-    owl_editwin_clear(e);
-    owl_editwin_adjust_for_locktext(e);
-  } else if (j==CTRL('a')) {
-    e->buffx=0;
-    owl_editwin_adjust_for_locktext(e);
-  } else if (j==CTRL('e')) {
-    e->buffx=owl_editwin_get_numchars_on_line(e, e->buffy);
-  } else if (j==CTRL('k')) {
-    owl_editwin_delete_to_endofline(e);
-  } else if (j>127 || ((j<32) && (j!=10) && (j!=13))) {
+  if (j>127 || ((j<32) && (j!=10) && (j!=13))) {
     return;
   } else {
     owl_editwin_insert_char(e, j);
   }
-
-  /* check if we need to scroll down on enter */
-  if (j==13 || j==10) {
-    if (e->buffy-e->topline >= e->winlines) {
-      e->topline+=e->winlines/2;
-    }
-  }
-
-  owl_editwin_redisplay(e, 0);
 }
 
-char *owl_editwin_get_text(owl_editwin *e) {
+char *owl_editwin_get_text(owl_editwin *e)
+{
   return(e->buff+e->lock);
 }
 
-int owl_editwin_get_numchars_on_line(owl_editwin *e, int line) {
+int owl_editwin_get_numchars_on_line(owl_editwin *e, int line)
+{
   int i;
   char *ptr1, *ptr2;
 
@@ -770,7 +931,8 @@ int owl_editwin_get_numchars_on_line(owl_editwin *e, int line) {
   return(ptr2-ptr1); /* don't count the newline for now */
 }
 
-int owl_editwin_get_numlines(owl_editwin *e) {
+int owl_editwin_get_numlines(owl_editwin *e)
+{
   return(owl_text_num_lines(e->buff));
 }
 
