@@ -1,6 +1,6 @@
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: keys.c,v 1.14 2002/09/16 18:51:33 kretch Exp $";
+static const char fileIdent[] = "$Id: keys.c,v 1.20 2003/06/30 22:12:41 kretch Exp $";
 
 #define BIND_CMD(kpress, command, desc) \
          owl_keymap_create_binding(km, kpress, command, NULL, desc);
@@ -125,6 +125,21 @@ void owl_keys_setup_keymaps(owl_keyhandler *kh) {
   BIND_CMD("LF",          "editline:done", "executes the command");
   BIND_CMD("CR",          "editline:done", "executes the command");
 
+  
+  /****************************************************************/
+  /**************************** EDITRESPONSE **********************/
+  /****************************************************************/
+
+  km_ew_onel = km = owl_keyhandler_create_and_add_keymap(kh, "editresponse",
+       "Single-line response to question", 
+       owl_keys_editwin_default, NULL, owl_keys_editwin_postalways);
+  owl_keymap_set_submap(km_ew_onel, km_editwin);
+
+  BIND_CMD("C-u",         "edit:delete-all", "Clears the entire line");
+
+  BIND_CMD("LF",          "editresponse:done", "executes the command");
+  BIND_CMD("CR",          "editresponse:done", "executes the command");
+
 
   /****************************************************************/
   /**************************** POPLESS ***************************/
@@ -240,13 +255,17 @@ void owl_keys_setup_keymaps(owl_keyhandler *kh) {
   BIND_CMD("C-l",       "redisplay",       "");
 
   BIND_CMD("i",   "info",             "");
-  BIND_CMD("l",   "zlist",            "");
+  BIND_CMD("l",   "blist",            "");
+  BIND_CMD("B",   "alist",            "");
   BIND_CMD("M",   "pop-message",      "");
   BIND_CMD("T",   "delete trash",     "mark all 'trash' messages for deletion");
 
+  BIND_CMD("o",   "toggle-oneline", "");
+
   BIND_CMD("A",   "zaway toggle",     "toggles zaway on and off");
 
   BIND_CMD("z",   "start-command zwrite ", "start a zwrite command");
+  BIND_CMD("a",   "start-command aimwrite ", "start an aimwrite command");
   BIND_CMD("r",   "reply",            "reply to the current message");
   BIND_CMD("R",   "reply sender",     "reply to sender of the current message");
   BIND_CMD("C-r", "reply -e",         "reply to the current message, but allow editing of recipient");
