@@ -1,22 +1,34 @@
 #ifndef INC_OWL_H
 #define INC_OWL_H
 
-#include <zephyr/zephyr.h>
 #include <curses.h>
 #include <sys/param.h>
 #include <EXTERN.h>
 #include <netdb.h>
 #include <regex.h>
+#include <time.h>
+#include <libfaim/aim.h>
 #include "config.h"
+#ifdef HAVE_LIBZEPHYR
+#include <zephyr/zephyr.h>
+#endif
+#ifdef HAVE_COM_ERR_H
+#include <com_err.h>
+#endif
 
-static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.26 2003/02/25 17:56:22 kretch Exp $";
 
-#define OWL_VERSION         1.2.9
-#define OWL_VERSION_STRING "1.2.9"
+
+static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.67 2003/10/10 23:42:29 kretch Exp $";
+
+#define OWL_VERSION         2.0.8
+#define OWL_VERSION_STRING "2.0.8"
 
 #define OWL_DEBUG 0
 #define OWL_DEBUG_FILE "/var/tmp/owldebug"
 
+#define OWL_CONFIG_DIR "/.owl"             /* this is relative to the user's home directory */
+#define OWL_STARTUP_FILE "/.owl/startup"   /* this is relative to the user's home directory */
+
 #define OWL_FMTEXT_ATTR_NONE      0
 #define OWL_FMTEXT_ATTR_BOLD      1
 #define OWL_FMTEXT_ATTR_REVERSE   2
@@ -35,6 +47,13 @@ static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.26 2003/02/25 17:56:22 kre
 #define OWL_EDITWIN_STYLE_MULTILINE 0
 #define OWL_EDITWIN_STYLE_ONELINE   1
 
+#define OWL_PROTOCOL_ZEPHYR         0
+#define OWL_PROTOCOL_AIM            1
+#define OWL_PROTOCOL_JABBER         2
+#define OWL_PROTOCOL_ICQ            3
+#define OWL_PROTOCOL_YAHOO          4
+#define OWL_PROTOCOL_MSN            5
+
 #define OWL_MESSAGE_TYPE_ADMIN      0
 #define OWL_MESSAGE_TYPE_GENERIC    1
 #define OWL_MESSAGE_TYPE_ZEPHYR     2
@@ -59,6 +78,9 @@ static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.26 2003/02/25 17:56:22 kre
 #define OWL_SCROLLMODE_PAGED       4
 #define OWL_SCROLLMODE_PAGEDCENTER 5
 
+#define OWL_STYLE_TYPE_INTERNAL  0
+#define OWL_STYLE_TYPE_PERL      1
+
 #define OWL_TAB               3  /* This *HAS* to be the size of TABSTR below */
 #define OWL_TABSTR        "   "
 #define OWL_MSGTAB            7
@@ -79,9 +101,10 @@ static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.26 2003/02/25 17:56:22 kre
 #define OWL_CTX_RECWIN       0x0f00
 #define OWL_CTX_RECV         0x0100
 #define OWL_CTX_TYPWIN       0xf000
-#define OWL_CTX_EDIT         0x3000
+#define OWL_CTX_EDIT         0x7000
 #define OWL_CTX_EDITLINE     0x1000
 #define OWL_CTX_EDITMULTI    0x2000
+#define OWL_CTX_EDITRESPONSE 0x4000
 
 #define OWL_USERCLUE_NONE       0
 #define OWL_USERCLUE_CLASSES    1
@@ -111,13 +134,17 @@ static const char owl_h_fileIdent[] = "$Id: owl.h,v 1.26 2003/02/25 17:56:22 kre
 
 #define OWL_CMD_ALIAS_SUMMARY_PREFIX "command alias to: "
 
-#ifndef CTRL
-#define CTRL(key) ((key)&037)
-#endif
-#ifndef META
-#define META(key) ((key)|0200)
+#define OWL_WEBZEPHYR_PRINCIPAL "daemon.webzephyr"
+#define OWL_WEBZEPHYR_CLASS     "webzephyr"
+#define OWL_WEBZEPHYR_OPCODE    "webzephyr"
+
+#if defined(HAVE_DES_STRING_TO_KEY) && defined(HAVE_DES_KEY_SCHED) && defined(HAVE_DES_ECB_ENCRYPT)
+#define OWL_ENABLE_ZCRYPT 1
 #endif
 
+#define OWL_META(key) ((key)|0200)
+/* OWL_CTRL is definied in kepress.c */
+
 #define LINE 2048
 
 typedef struct _owl_variable {
@@ -219,11 +246,12 @@ typedef struct _owl_cmd {	/* command */
 
 
 typedef struct _owl_zwrite {
-  char class[LINE];
-  char inst[LINE];
-  char realm[LINE];
-  char opcode[LINE];
+  char *class;
+  char *inst;
+  char *realm;
+  char *opcode;
   char *zsig;
+  char *message;
   owl_list recips;
   int cc;
   int noping;
@@ -238,15 +266,26 @@ typedef struct _owl_message {
   int id;
   int type;
   int direction;
+#ifdef HAVE_LIBZEPHYR
   ZNotice_t notice;
-  owl_fmtext fmtext;
+#endif
+  owl_fmtext fmtext;              /* this is now only a CACHED copy */
+  int invalid_format;             /* indicates whether fmtext needs to be regenerated */
   int delete;
-  char hostname[MAXHOSTNAMELEN];
-  owl_list attributes; /* this is a list of pairs */
+  char hostname[MAXHOSTNAMELEN+1];
+  owl_list attributes;            /* this is a list of pairs */
   char *time;
   char *zwriteline;
 } owl_message;
 
+typedef struct _owl_style {
+  char *name;
+  char *description;
+  int type;
+  char *perlfuncname;
+  void (*formatfunc) (owl_fmtext *fm, owl_message *m);
+} owl_style;
+
 typedef struct _owl_mainwin {
   int curtruncated;
   int lasttruncated;
@@ -297,8 +336,10 @@ typedef struct _owl_filter {
 } owl_filter;
 
 typedef struct _owl_view {
+  char *name;
   owl_filter *filter;
   owl_messagelist ml;
+  owl_style *style;
 } owl_view;
 
 typedef struct _owl_history {
@@ -320,6 +361,7 @@ typedef struct _owl_editwin {
   int style;
   int lock;
   int dotsend;
+  int echochar;
 } owl_editwin;
 
 typedef struct _owl_keybinding {
@@ -348,6 +390,23 @@ typedef struct _owl_keyhandler {
   int       kpstackpos;		/* location in stack (-1 = none) */
 } owl_keyhandler;
 
+typedef struct _owl_buddy {
+  int proto;
+  char *name;
+  int isidle;
+  int idlesince;
+} owl_buddy;
+
+typedef struct _owl_buddylist {
+  owl_list buddies;
+} owl_buddylist;
+
+typedef struct _owl_timer {
+  int direction;
+  time_t starttime;
+  int start;
+} owl_timer;
+
 typedef struct _owl_global {
   owl_mainwin mw;
   owl_popwin pw;
@@ -370,20 +429,20 @@ typedef struct _owl_global {
   int resizepending;
   int recwinlines;
   int typwinactive;
-  char thishost[LINE];
-  char homedir[LINE];
+  char *thishost;
+  char *homedir;
   int direction;
   int zaway;
   char *cur_zaway_msg;
   int haveconfig;
   int config_format;
-  char buffercommand[1024];
+  char *buffercommand;
   owl_editwin tw;
   owl_viewwin vw;
   void *perl;
   int debug;
   int starttime;
-  char startupargs[LINE];
+  char *startupargs;
   int userclue;
   int nextmsgid;
   int hascolors;
@@ -395,15 +454,31 @@ typedef struct _owl_global {
   owl_filterelement fe_true;
   owl_filterelement fe_false;
   owl_filterelement fe_null;
+  aim_session_t aimsess;
+  aim_conn_t waitingconn;
+  owl_timer aim_noop_timer;
+  owl_timer aim_ignorelogin_timer;
+  owl_timer aim_buddyinfo_timer;
+  int aim_loggedin;         /* true if currently logged into AIM */
+  int aim_doprocessing;     /* true if we should process AIM events (like pending login) */
+  char *aim_screenname;     /* currently logged in AIM screen name */
+  owl_buddylist buddylist;  /* list of logged in AIM buddies */
+  owl_list messagequeue;    /* for queueing up aim and other messages */
+  owl_dict styledict;       /* global dictionary of available styles */
+  char *response;           /* response to the last question asked */
+  int havezephyr;
+  int haveaim;
 } owl_global;
 
 /* globals */
-owl_global g;
+extern owl_global g;
 
 #include "owl_prototypes.h"
 
 /* these are missing from the zephyr includes for some reason */
+#ifdef HAVE_LIBZEPHYR
 int ZGetSubscriptions(ZSubscription_t *, int *);
 int ZGetLocations(ZLocations_t *,int *);
+#endif
 
 #endif /* INC_OWL_H */
