@@ -1,8 +1,9 @@
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: popwin.c,v 1.2 2002/06/28 06:18:46 nygren Exp $";
+static const char fileIdent[] = "$Id: popwin.c,v 1.3 2003/06/09 14:54:18 kretch Exp $";
 
-int owl_popwin_init(owl_popwin *pw) {
+int owl_popwin_init(owl_popwin *pw)
+{
   pw->active=0;
   pw->needsfirstrefresh=0;
   pw->lines=0;
@@ -10,7 +11,8 @@ int owl_popwin_init(owl_popwin *pw) {
   return(0);
 }
 
-int owl_popwin_up(owl_popwin *pw) {
+int owl_popwin_up(owl_popwin *pw)
+{
   int glines, gcols, startcol, startline;
 
   /* calculate the size of the popwin */
@@ -49,7 +51,8 @@ int owl_popwin_up(owl_popwin *pw) {
   return(0);
 }
 
-int owl_popwin_display_text(owl_popwin *pw, char *text) {
+int owl_popwin_display_text(owl_popwin *pw, char *text)
+{
   waddstr(pw->popwin, text);
   wnoutrefresh(pw->popwin);
   touchwin(pw->borderwin);
@@ -58,7 +61,8 @@ int owl_popwin_display_text(owl_popwin *pw, char *text) {
   return(0);
 }	      
 
-int owl_popwin_close(owl_popwin *pw) {
+int owl_popwin_close(owl_popwin *pw)
+{
   delwin(pw->popwin);
   delwin(pw->borderwin);
   pw->active=0;
@@ -68,13 +72,15 @@ int owl_popwin_close(owl_popwin *pw) {
   return(0);
 }
 
-int owl_popwin_is_active(owl_popwin *pw) {
+int owl_popwin_is_active(owl_popwin *pw)
+{
   if (pw->active==1) return(1);
   return(0);
 }
 
-int owl_popwin_refresh(owl_popwin *pw) {
-  /* this will refresh the border as well as the text area */
+/* this will refresh the border as well as the text area */
+int owl_popwin_refresh(owl_popwin *pw)
+{
   touchwin(pw->borderwin);
   touchwin(pw->popwin);
 
@@ -84,31 +90,38 @@ int owl_popwin_refresh(owl_popwin *pw) {
   return(0);
 }
 
-void owl_popwin_set_handler(owl_popwin *pw, void (*func)(int ch)) {
+void owl_popwin_set_handler(owl_popwin *pw, void (*func)(int ch))
+{
   pw->handler=func;
 }
 
-void owl_popwin_unset_handler(owl_popwin *pw) {
+void owl_popwin_unset_handler(owl_popwin *pw)
+{
   pw->handler=NULL;
 }
 
-WINDOW *owl_popwin_get_curswin(owl_popwin *pw) {
+WINDOW *owl_popwin_get_curswin(owl_popwin *pw)
+{
   return(pw->popwin);
 }
 
-int owl_popwin_get_lines(owl_popwin *pw) {
+int owl_popwin_get_lines(owl_popwin *pw)
+{
   return(pw->lines-2);
 }
 
-int owl_popwin_get_cols(owl_popwin *pw) {
+int owl_popwin_get_cols(owl_popwin *pw)
+{
   return(pw->cols-2);
 }
 
-int owl_popwin_needs_first_refresh(owl_popwin *pw) {
+int owl_popwin_needs_first_refresh(owl_popwin *pw)
+{
   if (pw->needsfirstrefresh) return(1);
   return(0);
 }
 
-void owl_popwin_no_needs_first_refresh(owl_popwin *pw) {
+void owl_popwin_no_needs_first_refresh(owl_popwin *pw)
+{
   pw->needsfirstrefresh=0;
 }
