@@ -2,13 +2,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <malloc.h>
 #include <unistd.h>
 #include <ctype.h>
 
-static const char fileIdent[] = "$Id: util.c,v 1.19 2003/02/21 18:14:00 kretch Exp $";
+static const char fileIdent[] = "$Id: util.c,v 1.33 2003/07/06 22:42:34 nygren Exp $";
 
-void sepbar(char *in) {
+void sepbar(char *in)
+{
   char buff[1024];
   WINDOW *sepwin;
   owl_messagelist *ml;
@@ -24,6 +24,12 @@ void sepbar(char *in) {
   wattron(sepwin, A_REVERSE);
   whline(sepwin, ACS_HLINE, owl_global_get_cols(&g));
 
+  if (owl_global_is_sepbar_disable(&g)) {
+    getyx(sepwin, y, x);
+    wmove(sepwin, y, owl_global_get_cols(&g)-1);
+    return;
+  }
+
   wmove(sepwin, 0, 2);  
 
   if (owl_messagelist_get_size(ml)==0) {
@@ -111,13 +117,15 @@ void sepbar(char *in) {
 }
 
 
-void pophandler_quit(int ch) {
+void pophandler_quit(int ch)
+{
   if (ch=='q') {
     owl_popwin_close(owl_global_get_popwin(&g));
   }
 }
 
-char **atokenize(char *buffer, char *sep, int *i) {
+char **atokenize(char *buffer, char *sep, int *i)
+{
   /* each element of return must be freed by user */
   char **args;
   char *workbuff, *foo;
@@ -165,7 +173,8 @@ char *skiptokens(char *buff, int n) {
   return buff;
 }
 
-void atokenize_free(char **tok, int nels) {
+void atokenize_free(char **tok, int nels)
+{
   int i;
   for (i=0; i<nels; i++) {
     owl_free(tok[i]);
@@ -174,7 +183,8 @@ void atokenize_free(char **tok, int nels) {
 }
 
 
-void owl_parsefree(char **argv, int argc) {
+void owl_parsefree(char **argv, int argc)
+{
   int i;
 
   if (!argv) return;
@@ -185,7 +195,8 @@ void owl_parsefree(char **argv, int argc) {
   owl_free(argv);
 }
 
-char **owl_parseline(char *line, int *argc) {
+char **owl_parseline(char *line, int *argc)
+{
   /* break a command line up into argv, argc.  The caller must free
      the returned values.  If there is an error argc will be set to
      -1, argv will be NULL and the caller does not need to free
@@ -268,11 +279,32 @@ char **owl_parseline(char *line, int *argc) {
   return(argv);
 }
 
+/* caller must free the return */
+char *owl_util_minutes_to_timestr(int in)
+{
+  int days, hours;
+  long run;
+  char *out;
+
+  run=in;
+
+  days=run/1440;
+  run-=days*1440;
+  hours=run/60;
+  run-=hours*60;
+
+  if (days>0) {
+    out=owl_sprintf("%i d %2.2i:%2.2i", days, hours, run);
+  } else {
+    out=owl_sprintf("    %2.2i:%2.2i", hours, run);
+  }
+  return(out);
+}
 
 
-int owl_util_find_trans(char *in, int len) {
-  /* return the index of the last char before a change from the first
-     one */
+/* return the index of the last char before a change from the first one */
+int owl_util_find_trans(char *in, int len)
+{
   int i;
   for (i=1; i<len; i++) {
     if (in[i] != in[0]) return(i-1);
@@ -281,15 +313,18 @@ int owl_util_find_trans(char *in, int len) {
 }
 
 
-void downstr(char *foo) {
+/* downcase the string 'foo' */
+void downstr(char *foo)
+{
   int i;
   for (i=0; foo[i]!='\0'; i++) {
     foo[i]=tolower(foo[i]);
   }
 }
 
-char *stristr(char *a, char *b) {
-  /* exactly like strstr but it's case insensitive */
+/* exactly like strstr but case insensitive */
+char *stristr(char *a, char *b)
+{
   char *x, *y, *ret;
 
   if ((x=owl_strdup(a))==NULL) return(NULL);
@@ -308,13 +343,14 @@ char *stristr(char *a, char *b) {
   return(ret);
 }
 
-char *owl_util_uniq(char *A, char *B, char *prohibit) {
-  /* Caller must free response. 
-     Takes in strings which are space-separated lists of tokens
-     and returns a single string containing no token more than once.
-     If prohibit is non-null, no token may start with a character
-     in prohibit.
-  */
+/* Caller must free response. 
+ * Takes in strings which are space-separated lists of tokens
+ * and returns a single string containing no token more than once.
+ * If prohibit is non-null, no token may start with a character
+ * in prohibit.
+ */
+char *owl_util_uniq(char *A, char *B, char *prohibit)
+{
   
   char *cat, **tok;
   int toklen, i, j, first=1;
@@ -341,9 +377,9 @@ char *owl_util_uniq(char *A, char *B, char *prohibit) {
   return(cat);
 }
 
-int only_whitespace(char *s) {
-  /* returns if a string is only whitespace */
-
+/* return 1 if a string is only whitespace, otherwise 0 */
+int only_whitespace(char *s)
+{
   int i;
   for (i=0; s[i]; i++) {
     if (!isspace((int) s[i])) return(0);
@@ -353,28 +389,33 @@ int only_whitespace(char *s) {
 
 /* hooks for doing memory allocation et. al. in owl */
 
-void *owl_malloc(size_t size) {
+void *owl_malloc(size_t size)
+{
   return(malloc(size));
 }
 
-void owl_free(void *ptr) {
+void owl_free(void *ptr)
+{
   free(ptr);
 }
 
-char *owl_strdup(const char *s1) {
+char *owl_strdup(const char *s1)
+{
   return(strdup(s1));
 }
 
-void *owl_realloc(void *ptr, size_t size) {
+void *owl_realloc(void *ptr, size_t size)
+{
   return(realloc(ptr, size));
 }
 
-char *owl_sprintf(const char *fmt, ...) {
-  /* allocates memory and returns the string or null.
-   * caller must free the string. 
-   * from Linux sprintf man page. 
-   */
-  
+
+/* allocates memory and returns the string or null.
+ * caller must free the string. 
+ * from Linux sprintf man page. 
+ */
+char *owl_sprintf(const char *fmt, ...)
+{
   int n, size = 100;
   char *p;
   va_list ap;
@@ -397,38 +438,44 @@ char *owl_sprintf(const char *fmt, ...) {
   }
 }
 
-char *short_zuser(char *in) {
-  /* the caller must free the return */
+/* Strip a local realm fron the zephyr user name.
+ * The caller must free the return
+ */
+char *short_zuser(char *in)
+{
   char *out, *ptr;
 
   out=owl_strdup(in);
   ptr=strchr(out, '@');
   if (ptr) {
-    if (!strcasecmp(ptr+1, ZGetRealm())) {
+    if (!strcasecmp(ptr+1, owl_zephyr_get_realm())) {
       *ptr='\0';
     }
   }
   return(out);
 }
 
-
-char *long_zuser(char *in) {
-  /* the caller must free the return */
+/* Append a local realm to the zephyr user name if necessary.
+ * The caller must free the return.
+ */
+char *long_zuser(char *in)
+{
   char *ptr;
 
   if (NULL != (ptr=strchr(in, '@'))) {
     return owl_strdup(in);
   } else {
-    return owl_sprintf("%s@%s", in, ZGetRealm());
+    return owl_sprintf("%s@%s", in, owl_zephyr_get_realm());
   }
 }
 
 
-char *owl_util_smartstripped_user(char *in) {
-  /* strip out the instance from a zsender's principal.  Preserves the
-   * realm if present.  daemon.webzephyr is a special case.  The
-   * caller must free the return */
-
+/* strip out the instance from a zsender's principal.  Preserves the
+ * realm if present.  daemon.webzephyr is a special case.  The
+ * caller must free the return
+ */
+char *owl_util_smartstripped_user(char *in)
+{
   char *ptr, *realm, *out;
 
   out=owl_strdup(in);
@@ -443,11 +490,10 @@ char *owl_util_smartstripped_user(char *in) {
     /* There's a '.' but it's in the realm */
     return(out);
   }
-  if (!strncasecmp(in, "daemon.webzephyr", strlen("daemon.webzephyr"))) {
+  if (!strncasecmp(in, OWL_WEBZEPHYR_PRINCIPAL, strlen(OWL_WEBZEPHYR_PRINCIPAL))) {
     return(out);
   }
 
-
   /* remove the realm from ptr, but hold on to it */
   realm=strchr(out, '@');
   if (realm) realm[0]='\0';
@@ -466,7 +512,8 @@ char *owl_util_smartstripped_user(char *in) {
   return(out);
 }
 
-char *owl_getquoting(char *line) {
+char *owl_getquoting(char *line)
+{
   if (line[0]=='\0') return("'");
   if (strchr(line, '\'')) return("\"");
   if (strchr(line, '"')) return("'");
@@ -474,11 +521,14 @@ char *owl_getquoting(char *line) {
   return("");
 }
 
-char *owl_util_substitute(char *in, char *from, char *to) {
-  /* Caller must free returned string.
-   * Returns a string with any occurances of 'from' replaced with 'to'.
-   * Does not currently handle backslash quoting, but may in the future.
-   */
+
+
+/* Return a string with any occurances of 'from' replaced with 'to'.
+ * Does not currently handle backslash quoting, but may in the future.
+ * Caller must free returned string.
+ */
+char *owl_util_substitute(char *in, char *from, char *to)
+{
   
   char *out;
   int   outlen, tolen, fromlen, inpos=0, outpos=0;
@@ -506,9 +556,11 @@ char *owl_util_substitute(char *in, char *from, char *to) {
   return(out);
 }
 
-void owl_util_tr(char *buff, char a, char b) {
-  /* replace all instances of character a in buff with the character
-     b.  buff must be null terminated */
+/* replace all instances of character a in buff with the character
+ * b.  buff must be null terminated.
+ */
+void owl_util_tr(char *buff, char a, char b)
+{
   int i;
 
   owl_function_debugmsg("In: %s", buff);
@@ -518,7 +570,10 @@ void owl_util_tr(char *buff, char a, char b) {
   owl_function_debugmsg("Out: %s", buff);
 }
 
-int owl_util_string_to_color(char *color) {
+
+/* Return the owl color associated with the named color */
+int owl_util_string_to_color(char *color)
+{
   if (!strcasecmp(color, "black")) {
     return(OWL_COLOR_BLACK);
   } else if (!strcasecmp(color, "red")) {
@@ -541,7 +596,9 @@ int owl_util_string_to_color(char *color) {
   return(OWL_COLOR_DEFAULT);
 }
 
-char *owl_util_color_to_string(int color) {
+/* Return a string name of the given owl color */
+char *owl_util_color_to_string(int color)
+{
   if (color==OWL_COLOR_BLACK)   return("black");
   if (color==OWL_COLOR_RED)     return("red");
   if (color==OWL_COLOR_GREEN)   return("green");
@@ -554,8 +611,9 @@ char *owl_util_color_to_string(int color) {
   return("Unknown color");
 }
 
-char *owl_util_get_default_tty() {
-  /* call must free the return */
+/* Get the default tty name.  Caller must free the return */
+char *owl_util_get_default_tty()
+{
   char *out, *tmp;
 
   if (getenv("DISPLAY")) {
@@ -573,7 +631,9 @@ char *owl_util_get_default_tty() {
 }
 
 
-void owl_hack_animate() {
+/* Animation hack */
+void owl_hack_animate()
+{
   owl_messagelist *ml;
   owl_message *m;
   owl_fmtext *fm;
@@ -626,9 +686,11 @@ void owl_hack_animate() {
   }
 }
 
-char *owl_util_stripnewlines(char *in) {
-  /* strip leading and trailing new lines.
-     caller must free the return */
+/* strip leading and trailing new lines.  Caller must free the
+ * return.
+ */
+char *owl_util_stripnewlines(char *in)
+{
   
   char  *tmp, *ptr1, *ptr2, *out;
 
@@ -647,6 +709,79 @@ char *owl_util_stripnewlines(char *in) {
   return(out);
 }
 
+/* Delete the line matching "line" from the named file.  If no such
+ * line is found the file is left intact.  If backup==1 then create a
+ * backupfile containing the original contents.  This is an
+ * inefficient impelementation which reads the entire file into
+ * memory.
+ */
+void owl_util_file_deleteline(char *filename, char *line, int backup)
+{
+  char buff[LINE], *text;
+  char *backupfilename="";
+  FILE *file, *backupfile=NULL;
+  int size, newline;
+
+  /* open the file for reading */
+  file=fopen(filename, "r");
+  if (!file) {
+    owl_function_makemsg("Error opening file %s", filename);
+    return;
+  }
+
+  /* open the backup file for writing */
+  if (backup) {
+    backupfilename=owl_sprintf("%s.backup", filename);
+    backupfile=fopen(backupfilename, "w");
+    if (!backupfile) {
+      owl_function_makemsg("Error opening file %s for writing", backupfilename);
+      owl_free(backupfilename);
+      return;
+    }
+    owl_free(backupfilename);
+  }
+
+  /* we'll read the entire file into memory, minus the line we don't want and
+   * and at the same time create the backup file if necessary
+   */
+  text=owl_malloc(LINE);
+  strcpy(text, "");
+  size=LINE;
+  while (fgets(buff, LINE, file)!=NULL) {
+    /* strip the newline */
+    newline=0;
+    if (buff[strlen(buff)-1]=='\n') {
+      buff[strlen(buff)-1]='\0';
+      newline=1;
+    }
+    
+    /* if we don't match the line, add to saved text in memory */
+    if (strcasecmp(buff, line)) {
+      size+=LINE;
+      text=owl_realloc(text, size);
+      strcat(text, buff);
+      if (newline) strcat(text, "\n");
+    }
+
+    /* write to backupfile if necessary */
+    if (backup) fputs(buff, backupfile);
+  }
+  if (backup) fclose(backupfile);
+  fclose(file);
+
+  /* now rewrite the original file from memory */
+  file=fopen(filename, "w");
+  if (!file) {
+    owl_function_makemsg("WARNING: Error opening %s for writing.  Use %s to restore.", filename, backupfilename);
+    owl_function_beep();
+    owl_free(line);
+    return;
+  }
+
+  fputs(text, file);
+  fclose(file);
+}
+
 /**************************************************************************/
 /************************* REGRESSION TESTS *******************************/
 /**************************************************************************/
@@ -655,7 +790,8 @@ char *owl_util_stripnewlines(char *in) {
 
 #define FAIL_UNLESS(desc,pred) printf("\t%-4s: %s\n", (pred)?"ok":(numfailed++,"FAIL"), desc)
 
-int owl_util_regtest(void) {
+int owl_util_regtest(void)
+{
   int numfailed=0;
 
   printf("BEGIN testing owl_util\n");
