@@ -3,15 +3,71 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <string.h>
-#include <zephyr/zephyr.h>
-#include <com_err.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: zephyr.c,v 1.15 2003/02/25 17:56:22 kretch Exp $";
+static const char fileIdent[] = "$Id: zephyr.c,v 1.24 2003/07/06 22:42:38 nygren Exp $";
 
+#ifdef HAVE_LIBZEPHYR
 Code_t ZResetAuthentication();
+#endif
 
-int owl_zephyr_loadsubs(char *filename) {
+int owl_zephyr_initialize()
+{
+#ifdef HAVE_LIBZEPHYR
+  int ret;
+  
+  if ((ret = ZInitialize()) != ZERR_NONE) {
+    com_err("owl",ret,"while initializing");
+    return(1);
+  }
+  if ((ret = ZOpenPort(NULL)) != ZERR_NONE) {
+    com_err("owl",ret,"while opening port");
+    return(1);
+  }
+#endif
+  return(0);
+}
+
+
+int owl_zephyr_shutdown()
+{
+#ifdef HAVE_LIBZEPHYR
+  unsuball();
+  ZClosePort();
+#endif
+  return(0);
+}
+
+int owl_zephyr_zpending()
+{
+#ifdef HAVE_LIBZEPHYR
+  return(ZPending());
+#else
+  return(0);
+#endif
+}
+
+char *owl_zephyr_get_realm()
+{
+#ifdef HAVE_LIBZEPHYR
+  return(ZGetRealm());
+#else
+  return("");
+#endif
+}
+
+char *owl_zephyr_get_sender()
+{
+#ifdef HAVE_LIBZEPHYR
+  return(ZGetSender());
+#else
+  return("");
+#endif
+}
+
+int owl_zephyr_loadsubs(char *filename)
+{
+#ifdef HAVE_LIBZEPHYR
   /* return 0  on success
    *        -1 on file error
    *        -2 on subscription error
@@ -81,9 +137,14 @@ int owl_zephyr_loadsubs(char *filename) {
   }
 
   return(ret);
+#else
+  return(0);
+#endif
 }
 
-int owl_zephyr_loadloginsubs(char *filename) {
+int owl_zephyr_loadloginsubs(char *filename)
+{
+#ifdef HAVE_LIBZEPHYR
   FILE *file;
   ZSubscription_t subs[3001];
   char subsfile[1024], buffer[1024];
@@ -136,9 +197,14 @@ int owl_zephyr_loadloginsubs(char *filename) {
   }
 
   return(ret);
+#else
+  return(0);
+#endif
 }
 
-void unsuball() {
+void unsuball()
+{
+#if HAVE_LIBZEPHYR
   int ret;
 
   ZResetAuthentication();
@@ -146,9 +212,12 @@ void unsuball() {
   if (ret != ZERR_NONE) {
     com_err("owl",ret,"while unsubscribing");
   }
+#endif
 }
 
-int owl_zephyr_sub(char *class, char *inst, char *recip) {
+int owl_zephyr_sub(char *class, char *inst, char *recip)
+{
+#ifdef HAVE_LIBZEPHYR
   ZSubscription_t subs[5];
   int ret;
 
@@ -162,10 +231,15 @@ int owl_zephyr_sub(char *class, char *inst, char *recip) {
     ret=-2;
   }
   return(0);
+#else
+  return(0);
+#endif
 }
 
 
-int owl_zephyr_unsub(char *class, char *inst, char *recip) {
+int owl_zephyr_unsub(char *class, char *inst, char *recip)
+{
+#ifdef HAVE_LIBZEPHYR
   ZSubscription_t subs[5];
   int ret;
 
@@ -179,10 +253,14 @@ int owl_zephyr_unsub(char *class, char *inst, char *recip) {
     ret=-2;
   }
   return(0);
+#else
+  return(0);
+#endif
 }
 
-
-char *owl_zephyr_get_field(ZNotice_t *n, int j, int *k) {
+#ifdef HAVE_LIBZEPHYR
+char *owl_zephyr_get_field(ZNotice_t *n, int j, int *k)
+{
   /* return a pointer to the Jth field, place the length in k.  If the
      field doesn't exist return an emtpy string */
   int i, count, save;
@@ -209,9 +287,16 @@ char *owl_zephyr_get_field(ZNotice_t *n, int j, int *k) {
   *k=0;
   return("");
 }
+#else
+char *owl_zephyr_get_field(void *n, int j, int *k)
+{
+  return("");
+}
+#endif
 
-
-int owl_zephyr_get_num_fields(ZNotice_t *n) {
+#ifdef HAVE_LIBZEPHYR
+int owl_zephyr_get_num_fields(ZNotice_t *n)
+{
   int i, fields;
 
   fields=1;
@@ -221,9 +306,16 @@ int owl_zephyr_get_num_fields(ZNotice_t *n) {
   
   return(fields);
 }
+#else
+int owl_zephyr_get_num_fields(void *n)
+{
+  return(0);
+}
+#endif
 
-
-char *owl_zephyr_get_message(ZNotice_t *n, int *k) {
+#ifdef HAVE_LIBZEPHYR
+char *owl_zephyr_get_message(ZNotice_t *n, int *k)
+{
   /* return a pointer to the message, place the message length in k */
   if (!strcasecmp(n->z_opcode, "ping")) {
     *k=0;
@@ -232,9 +324,11 @@ char *owl_zephyr_get_message(ZNotice_t *n, int *k) {
 
   return(owl_zephyr_get_field(n, 2, k));
 }
+#endif
 
-
-char *owl_zephyr_get_zsig(ZNotice_t *n, int *k) {
+#ifdef HAVE_LIBZEPHYR
+char *owl_zephyr_get_zsig(ZNotice_t *n, int *k)
+{
   /* return a pointer to the zsig if there is one */
 
   if (n->z_message_len==0) {
@@ -244,9 +338,16 @@ char *owl_zephyr_get_zsig(ZNotice_t *n, int *k) {
   *k=strlen(n->z_message);
   return(n->z_message);
 }
+#else
+char *owl_zephyr_get_zsig(void *n, int *k)
+{
+  return("");
+}
+#endif
 
-
-int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *recipient, char *message) {
+int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *recipient, char *message)
+{
+#ifdef HAVE_LIBZEPHYR
   int ret;
   ZNotice_t notice;
     
@@ -282,19 +383,29 @@ int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *rec
     owl_function_makemsg("Error sending zephyr");
     return(ret);
   }
-
   return(0);
+#else
+  return(0);
+#endif
 }
 
-Code_t send_zephyr_helper(ZNotice_t *notice, char *buf, int len, int wait) {
+#ifdef HAVE_LIBZEPHYR
+Code_t send_zephyr_helper(ZNotice_t *notice, char *buf, int len, int wait)
+{
   return(ZSendPacket(buf, len, 0));
 }
+#endif
 
-void send_ping(char *to) {
+void send_ping(char *to)
+{
+#ifdef HAVE_LIBZEPHYR
   send_zephyr("PING", "", "MESSAGE", "PERSONAL", to, "");
+#endif
 }
 
-void owl_zephyr_handle_ack(ZNotice_t *retnotice) {
+#ifdef HAVE_LIBZEPHYR
+void owl_zephyr_handle_ack(ZNotice_t *retnotice)
+{
   char *tmp;
   
   /* if it's an HMACK ignore it */
@@ -329,16 +440,31 @@ void owl_zephyr_handle_ack(ZNotice_t *retnotice) {
     owl_function_makemsg("Internal error on ack (%s)", retnotice->z_message);
   }
 }
+#else
+void owl_zephyr_handle_ack(void *retnotice)
+{
+}
+#endif
 
-int owl_zephyr_notice_is_ack(ZNotice_t *n) {
+#ifdef HAVE_LIBZEPHYR
+int owl_zephyr_notice_is_ack(ZNotice_t *n)
+{
   if (n->z_kind == SERVNAK || n->z_kind == SERVACK || n->z_kind == HMACK) {
     if (!strcasecmp(n->z_class, LOGIN_CLASS)) return(0);
     return(1);
   }
   return(0);
 }
+#else
+int owl_zephyr_notice_is_ack(void *n)
+{
+  return(0);
+}
+#endif
   
-void owl_zephyr_zaway(owl_message *m) {
+void owl_zephyr_zaway(owl_message *m)
+{
+#ifdef HAVE_LIBZEPHYR
   char *tmpbuff, *myuser, *to;
   
   /* bail if it doesn't look like a message we should reply to.  Some
@@ -376,10 +502,12 @@ void owl_zephyr_zaway(owl_message *m) {
   /* display the message as an admin message in the receive window */
   owl_function_make_outgoing_zephyr(owl_global_get_zaway_msg(&g), tmpbuff, "Automated reply:");
   owl_free(tmpbuff);
+#endif
 }
 
-
-void owl_zephyr_hackaway_cr(ZNotice_t *n) {
+#ifdef HAVE_LIBZEPHYR
+void owl_zephyr_hackaway_cr(ZNotice_t *n)
+{
   /* replace \r's with ' '.  Gross-ish */
   int i;
 
@@ -389,8 +517,11 @@ void owl_zephyr_hackaway_cr(ZNotice_t *n) {
     }
   }
 }
+#endif
 
-void owl_zephyr_zlocate(char *user, char *out, int auth) {
+void owl_zephyr_zlocate(char *user, char *out, int auth)
+{
+#ifdef HAVE_LIBZEPHYR
   int ret, numlocs;
   int one = 1;
   ZLocations_t locations;
@@ -416,9 +547,12 @@ void owl_zephyr_zlocate(char *user, char *out, int auth) {
     sprintf(out, "%s%s: %s\t%s\t%s\n", out, myuser, locations.host, locations.tty, locations.time);
     owl_free(myuser);
   }
+#endif
 }
 
-void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip) {
+void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip)
+{
+#ifdef HAVE_LIBZEPHYR
   char *line, subsfile[LINE], buff[LINE];
   FILE *file;
 
@@ -458,79 +592,198 @@ void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip) {
   owl_function_makemsg("Subscription added");
   
   owl_free(line);
+#endif
 }
 
-void owl_zephyr_delsub(char *filename, char *class, char *inst, char *recip) {
-  char *line, subsfile[LINE], buff[LINE], *text;
-  char backupfilename[LINE];
-  FILE *file, *backupfile;
-  int size;
+void owl_zephyr_delsub(char *filename, char *class, char *inst, char *recip)
+{
+#ifdef HAVE_LIBZEPHYR
+  char *line, *subsfile;
   
   line=owl_zephyr_makesubline(class, inst, recip);
+  line[strlen(line)-1]='\0';
 
-  /* open the subsfile for reading */
-  if (filename==NULL) {
-    sprintf(subsfile, "%s/%s", owl_global_get_homedir(&g), ".zephyr.subs");
+  if (!filename) {
+    subsfile=owl_sprintf("%s/.zephyr.subs", owl_global_get_homedir(&g));
   } else {
-    strcpy(subsfile, filename);
+    subsfile=owl_strdup(filename);
   }
-  file=fopen(subsfile, "r");
-  if (!file) {
-    owl_function_makemsg("Error opening file %s", subsfile);
-    owl_free(line);
-    return;
-  }
-
-  /* open the backup file for writing */
-  sprintf(backupfilename, "%s.backup", subsfile);
-  backupfile=fopen(backupfilename, "w");
-  if (!backupfile) {
-    owl_function_makemsg("Error opening file %s for writing", backupfilename);
-    owl_free(line);
-    return;
-  }
-
-  /* we'll read the entire file into memory, minus the line we don't want and
-   * and at the same time create a backup file */
-  text=owl_malloc(LINE);
-  strcpy(text, "");
-  size=LINE;
-  while (fgets(buff, LINE, file)!=NULL) {
-    /* if we don't match the line, add to text */
-    if (strcasecmp(buff, line)) {
-      size+=LINE;
-      text=owl_realloc(text, size);
-      strcat(text, buff);
-    }
-
-    /* write to backupfile */
-    fputs(buff, backupfile);
-  }
-  fclose(backupfile);
-  fclose(file);
-
-  /* now open the original subs file for writing and write out the
-   * subs */
-  file=fopen(subsfile, "w");
-  if (!file) {
-    owl_function_makemsg("WARNING: Error opening %s to rewrite subscriptions.  Use %s to restore", subsfile, backupfilename);
-    owl_function_beep();
-    owl_free(line);
-    return;
-  }
-
-  fputs(text, file);
-  fclose(file);
+  
+  owl_util_file_deleteline(subsfile, line, 1);
+  owl_free(subsfile);
   owl_free(line);
-
   owl_function_makemsg("Subscription removed");
+#endif
 }
 
-char *owl_zephyr_makesubline(char *class, char *inst, char *recip) {
-  /* caller must free the return */
+/* caller must free the return */
+char *owl_zephyr_makesubline(char *class, char *inst, char *recip)
+{
   char *out;
 
   out=owl_malloc(strlen(class)+strlen(inst)+strlen(recip)+30);
   sprintf(out, "%s,%s,%s\n", class, inst, !strcmp(recip, "") ? "*" : recip);
   return(out);
 }
+
+
+void owl_zephyr_zlog_in(void)
+{
+#ifdef HAVE_LIBZEPHYR
+  char *exposure, *eset;
+  int ret;
+
+  ZResetAuthentication();
+    
+  eset=EXPOSE_REALMVIS;
+  exposure=ZGetVariable("exposure");
+  if (exposure==NULL) {
+    eset=EXPOSE_REALMVIS;
+  } else if (!strcasecmp(exposure,EXPOSE_NONE)) {
+    eset = EXPOSE_NONE;
+  } else if (!strcasecmp(exposure,EXPOSE_OPSTAFF)) {
+    eset = EXPOSE_OPSTAFF;
+  } else if (!strcasecmp(exposure,EXPOSE_REALMVIS)) {
+    eset = EXPOSE_REALMVIS;
+  } else if (!strcasecmp(exposure,EXPOSE_REALMANN)) {
+    eset = EXPOSE_REALMANN;
+  } else if (!strcasecmp(exposure,EXPOSE_NETVIS)) {
+    eset = EXPOSE_NETVIS;
+  } else if (!strcasecmp(exposure,EXPOSE_NETANN)) {
+    eset = EXPOSE_NETANN;
+  }
+   
+  ret=ZSetLocation(eset);
+  if (ret != ZERR_NONE) {
+    /*
+      char buff[LINE];
+      sprintf(buff, "Error setting location: %s", error_message(ret));
+      owl_function_makemsg(buff);
+    */
+  }
+#endif
+}
+
+void owl_zephyr_zlog_out(void)
+{
+#ifdef HAVE_LIBZEPHYR
+  int ret;
+
+  ZResetAuthentication();
+  ret=ZUnsetLocation();
+  if (ret != ZERR_NONE) {
+    /*
+      char buff[LINE];
+      sprintf(buff, "Error unsetting location: %s", error_message(ret));
+      owl_function_makemsg(buff);
+    */
+  }
+#endif
+}
+
+void owl_zephyr_addbuddy(char *name)
+{
+  char *filename;
+  FILE *file;
+  
+  filename=owl_sprintf("%s/.anyone", owl_global_get_homedir(&g));
+  file=fopen(filename, "a");
+  owl_free(filename);
+  if (!file) {
+    owl_function_makemsg("Error opening zephyr buddy file for append");
+    return;
+  }
+  fprintf(file, "%s\n", name);
+  fclose(file);
+}
+
+void owl_zephyr_delbuddy(char *name)
+{
+  char *filename;
+
+  filename=owl_sprintf("%s/.anyone", owl_global_get_homedir(&g));
+  owl_util_file_deleteline(filename, name, 0);
+  owl_free(filename);
+}
+
+/* return auth string */
+#ifdef HAVE_LIBZEPHYR
+char *owl_zephyr_get_authstr(ZNotice_t *n)
+{
+
+  if (!n) return("UNKNOWN");
+
+  if (n->z_auth == ZAUTH_FAILED) {
+    return ("FAILED");
+  } else if (n->z_auth == ZAUTH_NO) {
+    return ("NO");
+  } else if (n->z_auth == ZAUTH_YES) {
+    return ("YES");
+  } else {
+    return ("UNKNOWN");
+  }           
+}
+#else
+char *owl_zephyr_get_authstr(void *n)
+{
+  return("");
+}
+#endif
+
+/* returns a buffer of subscriptions or an error message.
+ * Caller must free the return.
+ */
+char *owl_zephyr_getsubs()
+{
+#ifdef HAVE_LIBZEPHYR
+  int ret, num, i, one;
+  ZSubscription_t sub;
+  char *out, *tmpbuff;
+  one=1;
+
+  ret=ZRetrieveSubscriptions(0, &num);
+  if (ret==ZERR_TOOMANYSUBS) {
+    out=owl_strdup("Zephyr: too many subscriptions\n");
+    return(out);
+  }
+
+  out=owl_malloc(num*500);
+  tmpbuff=owl_malloc(num*500);
+  strcpy(out, "");
+  for (i=0; i<num; i++) {
+    if ((ret = ZGetSubscriptions(&sub, &one)) != ZERR_NONE) {
+      owl_free(out);
+      owl_free(tmpbuff);
+      ZFlushSubscriptions();
+      out=owl_strdup("Error while getting subscriptions\n");
+      return(out);
+    } else {
+      sprintf(tmpbuff, "<%s,%s,%s>\n%s", sub.zsub_class, sub.zsub_classinst, sub.zsub_recipient, out);
+      strcpy(out, tmpbuff);
+    }
+  }
+
+  owl_free(tmpbuff);
+  ZFlushSubscriptions();
+  return(out);
+#else
+  return("");
+#endif
+}
+
+char *owl_zephyr_get_variable(char *var)
+{
+#ifdef HAVE_LIBZEPHYR
+  return(ZGetVariable(var));
+#else
+  return("");
+#endif
+}
+
+void owl_zephyr_set_locationinfo(char *host, char *val)
+{
+#ifdef HAVE_LIBZEPHYR
+  ZInitLocationInfo(host, val);
+#endif
+}
+  
